"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("tocca");
var ko = require("knockout");
var style_1 = require("./style");
var eventListener_1 = require("./eventListener");
var userAgent_1 = require("./userAgent");
var uiElementPropertyName = "uiElement";
var mouseEvents = {
    contextmenu: true,
    mousedown: true,
    mouseenter: true,
    mouseleave: true,
    mousemove: true,
    mouseout: true,
    mouseover: true,
    mouseup: true
};
var touchEvents = {
    touchcancel: true,
    touchend: true,
    touchmove: true,
    touchstart: true,
    longtap: true,
    swipeleft: true,
    swiperight: true,
    swipeup: true,
    swipedown: true
};
function isGlobalEvent(event) {
    return !(mouseEvents[event] || touchEvents[event]);
}
exports.settings = {
    tapDelay: 300
};
var observerOptions = {
    childList: true,
    subtree: true
};
var mutationObserver = new MutationObserver(function (mutationRecords) {
    mutationRecords.forEach(function (mutationRecord) {
        if (mutationRecord.removedNodes) {
            mutationRecord.removedNodes.forEach(function (removedNode) {
                var uiElement = removedNode[uiElementPropertyName];
                if (uiElement && uiElement.disposeOnDomRemoval && uiElement.dispose) {
                    uiElement.dispose();
                }
            });
        }
    });
});
mutationObserver.observe(document.body, observerOptions);
function setObserverOptions(options) {
    observerOptions = options;
}
exports.setObserverOptions = setObserverOptions;
var UiElement = (function () {
    function UiElement(tagName, elementType, options) {
        var _this = this;
        this.attributes = {};
        this.style = {};
        this.innerTextIsHtml = false;
        this.children = new Array();
        this.knockoutSubscriptions = Array();
        this.diposeOnDomRemoval = false;
        this.dispose = function () {
            _this.remove();
            for (var _i = 0, _a = _this.children; _i < _a.length; _i++) {
                var child = _a[_i];
                child.dispose();
            }
            _this.knockoutSubscriptions.forEach(function (subscription) {
                subscription.dispose();
            });
            if (_this.options.dispose) {
                _this.options.dispose();
            }
        };
        this.tagName = tagName;
        this.options = options || {};
        if (!this.options.elementType) {
            this.options.elementType = elementType;
        }
        this.innerTextIsHtml = this.options.innerTextIsHtml;
        if (this.options.innerText) {
            var innerText = this.options.innerText;
            this.innerText = ko.unwrap(innerText);
            if (ko.isObservable(innerText)) {
                innerText.subscribe(function (newValue) {
                    if (_this.innerTextIsHtml) {
                        setInterval(newValue);
                    }
                    else {
                        _this.setInnerText(newValue);
                    }
                });
            }
        }
        if (this.options.attributes) {
            this.options.attributes.forEach(function (attribute) {
                _this.attributes[attribute.name] = ko.unwrap(attribute.value);
                if (ko.isObservable(attribute.value)) {
                    _this.addSubscription(attribute.value, function (newValue) {
                        _this.setAttribute(attribute.name, newValue);
                    });
                }
            });
        }
        if (this.options.children) {
            var children = this.options.children;
            this.children = ko.unwrap(children);
            if (ko.isObservable(children)) {
                children.subscribe(function (newChildren) {
                    _this.setChildren(newChildren);
                });
            }
        }
        if (this.options.style) {
            var _loop_1 = function (styleProperty) {
                var styleValue = this_1.options.style[styleProperty];
                this_1.style[styleProperty] = ko.unwrap(styleValue);
                if (ko.isObservable(styleValue)) {
                    this_1.addSubscription(styleValue, function (newValue) {
                        var _a;
                        _this.setStyle((_a = {}, _a[styleProperty] = newValue, _a));
                    });
                }
            };
            var this_1 = this;
            for (var _i = 0, _a = Object.keys(this.options.style); _i < _a.length; _i++) {
                var styleProperty = _a[_i];
                _loop_1(styleProperty);
            }
        }
    }
    UiElement.prototype.addEventListenerToElement = function (type, listener, options) {
        if (!type) {
            throw new Error("type is mandatory");
        }
        if (!listener) {
            throw new Error("listener is mandatory");
        }
        if (!options) {
            options = {};
        }
        if (!options.passive) {
            options.passive = false;
        }
        if (!options.once) {
            options.once = false;
        }
        if (this.element.addEventListener) {
            this.element.addEventListener(type, function (event) {
                listener(event);
            }, {
                passive: options.passive,
                once: options.once
            });
        }
        else if (this.element.attachEvent) {
            this.element.attachEvent(type, function (event) {
                listener(event);
            });
        }
        else {
            throw "Your browser does not support 'addEventListener'";
        }
    };
    UiElement.prototype.build = function () {
        var _a;
        if (this.options.id) {
            this.element.id = this.options.id;
        }
        if (this.options.elementType) {
            this.setAttribute("elementtype", this.options.elementType);
        }
        var bindings = {
            style: this.style,
            attr: this.attributes,
        };
        if (this.innerText) {
            if (this.innerTextIsHtml) {
                bindings["html"] = this.innerText;
            }
            else {
                bindings["text"] = this.innerText;
            }
        }
        if (this.options.className) {
            if (ko.isObservable(this.options.className)) {
                bindings["css"] = this.options.className;
            }
            else {
                bindings["css"] = (_a = {},
                    _a[this.options.className] = true,
                    _a);
            }
        }
        ko.applyBindingsToNode(this.element, bindings);
        if (this.options.selectable === false) {
            this.addEventListenerToElement("selectstart", function () { return false; }, { passive: true });
            this.element.style.userSelect = "none";
        }
        if (this.children) {
            for (var _i = 0, _b = this.children; _i < _b.length; _i++) {
                var child = _b[_i];
                this.element.appendChild(child.render());
            }
        }
        if (this.options.addControlToDataDictionary) {
            if (!this.element.data) {
                this.element.data = {};
            }
            this.element.data.uiElement = this;
        }
    };
    UiElement.prototype.addSubscription = function (observable, callback) {
        this.knockoutSubscriptions.push(observable.subscribe(callback));
    };
    UiElement.prototype.render = function () {
        var _this = this;
        if (this.element) {
            this.remove();
        }
        if (typeof this.build === "function") {
            this.element = document.createElement(this.tagName);
            this.build();
            var clickEventListeners_1 = new Array();
            var doubleClickEventListeners_1 = new Array();
            if (this.options.eventListeners) {
                var eventListeners = this.options.eventListeners;
                eventListeners.forEach(function (eventListener) {
                    if (eventListener.type === "click") {
                        clickEventListeners_1.push(eventListener);
                        return;
                    }
                    if (eventListener.type === "dblclick") {
                        doubleClickEventListeners_1.push(eventListener);
                        return;
                    }
                    if (eventListener.type === "dragstart") {
                        _this.setAttribute("draggable", "true");
                    }
                    if (isGlobalEvent(eventListener.type)) {
                        _this.addEventListenerToElement(eventListener.type, function (event) {
                            if (eventListener.options) {
                                var options = eventListener.options;
                                var keyEvent = event;
                                if (options.altKey && !keyEvent.altKey) {
                                    return;
                                }
                                if (options.shiftKey && !keyEvent.shiftKey) {
                                    return;
                                }
                                if (options.ctrlKey && !keyEvent.ctrlKey) {
                                    return;
                                }
                                if (options.eventKey && options.eventKey !== keyEvent.key) {
                                    return;
                                }
                            }
                            eventListener.listener.call(_this, event);
                        }, eventListener.options);
                    }
                    else if (mouseEvents[eventListener.type] && userAgent_1.userAgent.device.supportsMouseEvents) {
                        _this.addEventListenerToElement(eventListener.type, eventListener.listener, eventListener.options);
                    }
                    else if (touchEvents[eventListener.type] && userAgent_1.userAgent.device.supportsTouchEvents) {
                        _this.addEventListenerToElement(eventListener.type, eventListener.listener, eventListener.options);
                    }
                });
                clickEventListeners_1.forEach(function (clickEventListener) {
                    if (userAgent_1.userAgent.device.supportsTouchEvents) {
                        _this.addEventListenerToElement("tap", clickEventListener.listener, clickEventListener.options);
                    }
                    if (userAgent_1.userAgent.device.supportsMouseEvents) {
                        _this.addEventListenerToElement(clickEventListener.type, clickEventListener.listener, clickEventListener.options);
                    }
                });
                doubleClickEventListeners_1.forEach(function (doubleClickEventListener) {
                    if (userAgent_1.userAgent.device.supportsTouchEvents) {
                        _this.addEventListenerToElement("dbltap", doubleClickEventListener.listener, doubleClickEventListener.options);
                    }
                    if (userAgent_1.userAgent.device.supportsMouseEvents) {
                        _this.addEventListenerToElement(doubleClickEventListener.type, doubleClickEventListener.listener, doubleClickEventListener.options);
                    }
                });
            }
            return this.element;
        }
        throw "The build method of this UiElement has not been defined";
    };
    UiElement.prototype.refresh = function () { };
    UiElement.prototype.remove = function () {
        if (this.element) {
            if (this.element.remove) {
                this.element.remove();
            }
            else {
                this.element.parentElement.removeChild(this.element);
            }
            this.element = null;
        }
    };
    UiElement.prototype.handleMessage = function (message) { };
    UiElement.prototype.getElement = function () {
        return this.element;
    };
    UiElement.prototype.focus = function (options) {
        if (this.element) {
            this.element.focus(options);
        }
    };
    UiElement.prototype.focusPreviousSibling = function (options) {
        if (this.element && this.element.previousElementSibling && this.element.nextSibling instanceof HTMLElement) {
            this.element.nextSibling.focus(options);
        }
    };
    UiElement.prototype.focusNextSibling = function (options) {
        if (this.element && this.element.previousElementSibling && this.element.nextSibling instanceof HTMLElement) {
            this.element.nextSibling.focus(options);
        }
    };
    UiElement.prototype.setInnerText = function (innerText) {
        this.innerText = innerText;
        if (this.element) {
            this.element.innerText = innerText;
        }
    };
    UiElement.prototype.setInnerHtml = function (innerHtml) {
        this.innerText = innerHtml;
        if (this.element) {
            this.element.innerHTML = innerHtml;
        }
    };
    UiElement.prototype.getAttribute = function (attributeName) {
        if (this.element) {
            return this.element.getAttribute(attributeName);
        }
        else {
            return ko.unwrap(this.attributes[attributeName]);
        }
    };
    UiElement.prototype.setAttribute = function (name, value) {
        var currentAttribute = this.attributes[name];
        if (currentAttribute && ko.isObservable(currentAttribute)) {
            currentAttribute(ko.unwrap(value));
        }
        else {
            this.attributes[name] = value;
            if (this.element) {
                this.element.setAttribute(name, value);
            }
        }
    };
    UiElement.prototype.deleteAttribute = function (attributeName) {
        if (this.attributes[attributeName]) {
            delete this.attributes[attributeName];
        }
        if (this.element) {
            this.element.removeAttribute(attributeName);
        }
    };
    UiElement.prototype.getStyle = function () {
        var cssProperties = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            cssProperties[_i] = arguments[_i];
        }
        if (!cssProperties || cssProperties.length === 0) {
            return this.style;
        }
        var style = {};
        for (var _a = 0, cssProperties_1 = cssProperties; _a < cssProperties_1.length; _a++) {
            var cssProperty = cssProperties_1[_a];
            if (this.style[cssProperty]) {
                style[cssProperty] = this.style[cssProperty];
            }
            else if (this.element) {
                style[cssProperty] = this.element.style[cssProperty];
            }
        }
        return style;
    };
    UiElement.prototype.getStyleValue = function (cssProperty) {
        if (this.style[cssProperty]) {
            return this.style[cssProperty];
        }
        else if (this.element) {
            return this.element.style[cssProperty];
        }
        return null;
    };
    UiElement.prototype.setStyle = function (newStyle, overwriteExisting) {
        var _this = this;
        if (newStyle) {
            Object.keys(newStyle).forEach(function (key) {
                if (_this.style.hasOwnProperty(key)) {
                    if (!overwriteExisting) {
                        return;
                    }
                }
                var newValue = newStyle[key];
                var currentValue = _this.style[key];
                if (currentValue && ko.isObservable(currentValue)) {
                    currentValue(ko.unwrap(newValue));
                }
                else {
                    _this.style[key] = newValue;
                    if (_this.element) {
                        if (newValue === null || newValue === undefined) {
                            if (userAgent_1.userAgent.browser.isInternetExplorer) {
                                _this.element.style[key] = style_1.defaultStyle[key];
                            }
                            else {
                                _this.element.style[key] = null;
                            }
                        }
                        else {
                            _this.element.style[key] = newValue;
                        }
                    }
                }
            });
        }
    };
    UiElement.prototype.setStyleNonStandard = function (name, value, overwriteExisting) {
        var _a;
        this.setStyle((_a = {}, _a[name] = value, _a), overwriteExisting);
    };
    UiElement.prototype.deleteStyle = function (style) {
        var styles;
        if (style instanceof Array) {
            styles = style;
        }
        else if (style instanceof Object) {
            styles = Object.keys(style);
        }
        else {
            styles = [style];
        }
        for (var _i = 0, styles_1 = styles; _i < styles_1.length; _i++) {
            var cssProperty = styles_1[_i];
            if (this.style && this.style.hasOwnProperty(cssProperty)) {
                delete this.style[cssProperty];
            }
            if (this.element) {
                this.element.style[cssProperty] = undefined;
            }
        }
    };
    UiElement.prototype.addEventListener = function (type, listener, options) {
        if (!this.options.eventListeners) {
            this.options.eventListeners = new Array();
        }
        this.options.eventListeners.push(new eventListener_1.EventListener(type, listener, options));
    };
    UiElement.prototype.dispatchEvent = function (type) {
        if (this.element) {
            return this.element.dispatchEvent(new Event(type));
        }
        return false;
    };
    UiElement.prototype.getChildren = function () {
        return this.children;
    };
    UiElement.prototype.setChildren = function (children, replace) {
        var _a;
        if (replace === void 0) { replace = true; }
        if (!children) {
            children = [];
        }
        if (replace) {
            this.children = children;
            if (this.element) {
                while (this.element.firstChild) {
                    this.element.removeChild(this.element.firstChild);
                }
            }
        }
        else {
            (_a = this.children).push.apply(_a, children);
        }
        if (this.element) {
            if (children.length > 0) {
                var documentFragment = document.createDocumentFragment();
                for (var _i = 0, children_1 = children; _i < children_1.length; _i++) {
                    var child = children_1[_i];
                    documentFragment.appendChild(child.render());
                }
                this.element.appendChild(documentFragment);
            }
        }
    };
    UiElement.prototype.addChild = function (newChild, referenceChild) {
        if (referenceChild) {
            var index = this.children.indexOf(referenceChild);
            if (index > -1) {
                this.children.splice(index, 0, newChild);
                if (this.element && referenceChild.getElement()) {
                    this.element.insertBefore(newChild.render(), referenceChild.getElement());
                }
            }
        }
        else {
            this.children.push(newChild);
            if (this.element) {
                this.element.appendChild(newChild.render());
            }
        }
    };
    UiElement.prototype.removeChild = function (element) {
        var index = this.children.indexOf(element);
        if (index > -1) {
            this.children.splice(index, 1);
            if (this.element) {
                this.element.removeChild(this.element.children[index]);
            }
        }
    };
    UiElement.prototype.empty = function () {
        this.children = [];
        if (this.element) {
            while (this.element.firstChild) {
                this.element.removeChild(this.element.firstChild);
            }
        }
    };
    return UiElement;
}());
exports.UiElement = UiElement;
//# sourceMappingURL=uiElement.js.map